#ifndef __BITMAPS_H
#define __BITMAPS_H

static unsigned char sbarrow_down_bits[] = {
//    0x00, 0x00, 0x7f, 0x7f, 0x3e, 0x1c, 0x08};
   0x7f, 0x3e, 0x3e, 0x1c, 0x1c, 0x08 };
static unsigned char sbarrow_left_bits[] = {
//  0x18, 0x1c, 0x1e, 0x1f, 0x1e, 0x1c, 0x18};
   0x20, 0x38, 0x3e, 0x3f, 0x3e, 0x38, 0x20 };
static unsigned char sbarrow_right_bits[] = {
//  0x0c, 0x1c, 0x3c, 0x7c, 0x3c, 0x1c, 0x0c};
   0x01, 0x07, 0x1f, 0x3f, 0x1f, 0x07, 0x01 };
static unsigned char sbarrow_up_bits[] = {
//   0x08, 0x1c, 0x3e, 0x7f, 0x7f, 0x00, 0x00};
   0x08, 0x1c, 0x1c, 0x3e, 0x3e, 0x7f };
static unsigned char checkmark_bits[] = {
   0x40, 0x60, 0x71, 0x3b, 0x1f, 0x0e, 0x04 };

#endif


