#ifndef AQUARIUSBUTTON_H
#define AQUARIUSBUTTON_H

//#include <tqvariant.h>
#include <tqwidget.h>
#include <tqimage.h>

class TQPixmap;
class TQColor;

class AquariusButton : public TQWidget
{
    TQ_OBJECT

public:
    AquariusButton( TQPixmap &pixmap, TQWidget* parent = 0, const char* name = 0);
    ~AquariusButton();
    TQColor Color();

public slots:
    void setColor(TQColor c);
    
protected:
    TQPixmap pixmap;
    TQImage image;
    TQColor color;
    void tint(TQColor &c);
    void mousePressEvent( TQMouseEvent *e );
    void paintEvent( TQPaintEvent *e);


signals:
    void clicked();

};

#endif // AQUARIUSBUTTON_H 
