#ifndef _BAGHIRALINKDRAG_H_
#define _BAGHIRALINKDRAG_H_

#include <tqdragobject.h>
#include <tqstring.h>

class BaghiraLinkDrag : public TQDragObject
{
//    TQ_OBJECT
      
public:
   BaghiraLinkDrag(TQString title, TQString command, TQString icon, int index = -1, TQWidget* dragSource = 0);
   ~BaghiraLinkDrag();
   static bool decode( const TQMimeSource * e, TQString * title,  TQString * command, TQString * icon, int * index);
   static bool canDecode( const TQMimeSource * e);
   static void setAccepted( );
   virtual const char * format ( int i = 0 ) const;
   virtual TQByteArray encodedData ( const char * ) const;
   static bool accepted( );
private:
   TQByteArray a;
};

#endif
